<?php
/**
* Very simple gallery script
*
* Generates list of thumbnails for the supported image types in the current folder.
*
* Requires PHP installation and GD library.
*
* This version of the script was created for the sole reason to provide "quick'n'dirty"
* image gallery functionality the easiest possible way.
*
* @version 0.1
* @author Baglan Dosmagambetov <baglan@ankara.edu.tr>
* @copyright 2004, Baglan Dosmagambeov
*/

/*
* General definitions
*/
define('WIDTH',100);				// Thumbmail image width
define('HEIGHT',100);				// Thumbnail image height
define('EXT','.bdgallerythumb');	// Thumbnail extension

/*
* Is thumbnail image requested?
*/
if ( $_REQUEST['_a']=='thumb' )
{
	// Intialize frequently used request parameters
	$file = $_REQUEST['file'];
	$thumb = $_REQUEST['file'].EXT;

	/*
	* Check if a thumbnail with a newer timestamp exists.
	* If exists, spool the thumbnail
	*/
	if
	(
		file_exists($thumb)
		&& filemtime($thumb)>=filemtime($file)
	)
	{
		header('Content-type: image/jpeg');
		$size = filesize($_REQUEST['file'].EXT);
		$fd = fopen($_REQUEST['file'].EXT,'rb');
		echo fread( $fd, $size );
		fclose( $fd );
		exit();
	}

	/*
	* Thumbnail doesn't exist
	* a. Make sure that file exists and that it's of supported format
	* b. Create and spool the thumbnail
	* c. Save created thumbnail 
	*/
	$imageInfo = getimagesize( $file );
	switch( $imageInfo[2] )
	{
		case 1:	// GIF
			$image = imagecreatefromgif( $file );
			break;
		case 2:	// JPEG
			$image = imagecreatefromjpeg( $file );
			break;
		case 3:	// PNG
			$image = imagecreatefrompng( $file );
			break;
		default:
			/*
			* If this point is reached, the file requested does not exists or
			* is of unsupported format.
			* Bail out.
			*/
			exit();
	}

	// Create the thumnail image
	$result = imagecreatetruecolor(WIDTH,HEIGHT);
	$bg = imagecolorallocate( $result, 255, 255, 255 );
	imagefilledrectangle( $result, 0, 0, WIDTH-1, HEIGHT-1, $bg );

	// Calculate the aspect ratio for the thumbnail
	$ratio1 = $imageInfo[0]/WIDTH;
	$ratio2 = $imageInfo[1]/HEIGHT;
	$ratio = $ratio1>$ratio2?$ratio1:$ratio2;
	if ( $ratio<1 )
		$ratio=1;
	$width = $imageInfo[0]/$ratio;
	$height = $imageInfo[1]/$ratio;
	imagecopyresampled( $result, $image, (WIDTH-$width)/2, (HEIGHT-$height)/2, 0, 0, $width, $height, $imageInfo[0], $imageInfo[1]);
	
	// Spool thumbnail image
	header('Content-type: image/jpeg');
	imagejpeg($result);
	
	// Save the thumbnail image so that if requested again,thumbnail will not have to be generated
	imagejpeg($result,$thumb);
	
	// End program here
	exit();
}
/*
* END thumb
*/

/*
* By default, list of images is requested
*/

/*
* Get supported image types.
* Read-only GIF support somehow is not returned, see if corresponding function exists
*/
$supported = imagetypes();
if ( function_exists('imagecreatefromgif') )
	$supported |= IMG_GIF;

/*
* Get the list of files with supported extensions and their thumbnails in
* respective arrays
*/
$files = array();
$thumbs = array();
$dir = opendir('.');
while( ($file=readdir( $dir ))!==false )
{
	// Skip if not a file
	if ( !is_file($file) )
		continue;
		
	$ext = strrchr($file, '.');
	$extLower = strtolower($ext);
	if
	(
		$extLower=='.gif' && ($supported & IMG_GIF)
		|| $extLower=='.png' && ($supported & IMG_PNG)
		|| $extLower=='.jpeg' && ($supported & IMG_JPG)
		|| $extLower=='.jpg' && ($supported & IMG_JPG)
	)
		$files[] = $file;
	else if ( $ext==EXT )
		$thumbs[] = $file;
}
closedir($dir);

/*
* Create HTML representation
*/
foreach( $files as $file )
{
	/*
	* Title (text shown when pointer is over the thumbnail)
	* shall be in form:
	* <file_name> - <size>
	*/
	$title = $file;
	$size = filesize( $file );
	$measure = 'B';
	if ( $size>1024 )
	{
		$size /= 1024;
		$measure = 'KB';
		if ( $size>1024 )
		{
			$size /= 1024;
			$measure = 'MB';
		}
	}
	$title .= " - ".number_format($size,1).$measure;
?>
<a href="<?php echo $file ?>"><img src="index.php?_a=thumb&file=<?php echo $file ?>&<?php echo filemtime($file) ?>" border="1" title="<?php echo $title ?>" alt="<?php echo $title ?>"></a>
<?php
}

/*
* Remove thumbs not associated with image files
*/
$extLength = strlen( EXT );
foreach( $thumbs as $thumb )
{
	$file = substr( $thumb, 0, strlen($thumb)-$extLength );
	if ( !file_exists($file) )
		unlink($thumb);
}
?>